program Simplex_Method;

{$M $2000 $2000 $4000}

uses App, Views, Objects, Menus, KbCodes, Drivers, MsgBox, Dos, Memory,
     Gadgets, Help, Setup, IO, Crt, Dialogs;

const Sizes: array[1..5]of Byte=($54, $44, $53, $34, $35);

type
  TMainApp=object(TApplication)
    Pntr: TData;
    InputData: TInputData;
    OutputData: TOutputData;
    P: PDialog;
    T: PHelpWindow;
    Clock: PClockView;
    constructor Init;
    procedure InitMenuBar; virtual;
    procedure InitStatusLine; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure Idle; virtual;
    procedure Help;
    procedure Setup;
    procedure EnterData;
    procedure ExecuteProgram;
    procedure ConvertData;
    procedure MainSimplex;
    procedure About;
    procedure ReadData;
    procedure SetSizes(Siz: Byte);
  end;

var Data: array[1..5, 1..4] of Real;
    Fm: array[1..5] of Real;
    X, Y: Byte;

procedure TMainApp.Idle;
begin
  inherited Idle;
  Clock^.Update;
end;

procedure TMainApp.About;
begin
  MessageBox(#3+'-⮤'+#13+#3+'Copyright (c) Voinov Sergey 1998.',
             nil, mfInformation+mfOKButton);
end;

procedure TMainApp.InitMenuBar;
  var R: TRect;
begin
  GetExtent(R);
  R.A.Y:=R.B.Y-1;
  MenuBar := New(PMenuBar, Init(R, NewMenu(
    NewSubMenu('~~', 1, NewMenu(
      NewSubMenu('~~ணࠬ', 1, NewMenu(
        NewItem('~~ ', 'Alt+D', kbAltD, 1004, 0,
        NewItem('~C~-⮤', 'Alt+S', kbAltS, 1005, 0, nil))),
      NewItem('~~ன', 'Alt+N', kbAltN, 1003, 0,
      NewItem('~~ࠢ', 'F1', kbF1, 1002, 0,
      NewItem('~~믮...', 'Alt+E', kbAltE, 1001, 0,
      NewLine(
        NewItem('~~襭 ࠡ', 'Alt-X', kbAltX, cmQuit, hcExit, nil))))))),nil))));
end;

procedure TMainApp.InitStatusLine;
  var R: TRect;
begin
  GetExtent(R);
  R.B.Y:=R.A.Y+1;
  StatusLine:=New(PStatusLine, Init(R,
    NewStatusDef(0, 1,
    NewStatusKey('~F10~ ', kbF10, cmMenu,
    NewStatusKey('~Alt O~  ணࠬ', kbAltO, 1006, nil)), nil)));
end;

procedure TMainApp.MainSimplex;
  var R: TRect;
      f: Text;
      n, j: Integer;
begin
  for n:=1 to X do
    for j:=1 to Y-1 do
      Data[n][j]:=Data[n][j]/2;

  if Pntr.OutMode=1 then begin
    Assign(f, Pntr.OutName+'.txt');
    Rewrite(f);
    for j:=1 to Y do
      for n:=1 to X do
        Writeln(f, Trunc(Data[n][j]));
    Close(f);
  end
  else begin
    for n:=1 to X do
      for j:=1 to Y-1 do
        Str(Round(Data[n][j]), OutputData.S[(j-1)*X+n]);

    for j:=1 to X do
      Str(Round(Fm[j]), OutputData.S[(Y-1)*X+j]);

    P:=New(PIOWindow, Init(X, Y, '뢮 '));
    ExecuteDialog(P, @OutputData);
  end;
end;

procedure TMainApp.ExecuteProgram;
  var S: String;
      Error: Integer;
begin
  S:='';
  InputBox(' ணࠬ', '  ணࠬ: ', S, 80);
  if S<>'' then begin
    Exec(S, '');
    Error:=DosError;
    Str(Error, S);
    if Error<>0 then
      MessageBox(#3'訡 ᪠ ணࠬ #'+S, nil, mfError);
  end;
end;

procedure TMainApp.Setup;
  var R: TRect;
begin
  P:=New(PSetupDialog, Init);
  ExecuteDialog(P, @Pntr);
  ReadData;
  SetSizes(Sizes[Pntr.Size+1]);
end;

procedure TMainApp.EnterData;
begin
  P:=New(PIOWindow, Init(X, Y, ' '));
  ExecuteDialog(P, @InputData);
  ConvertData;
end;

procedure TMainApp.HandleEvent(var Event: TEvent);
begin
  inherited HandleEvent(Event);
  case Event.Command of
    1001: ExecuteProgram;
    1002: Help;
    1003: Setup;
    1004: EnterData;
    1005: MainSimplex;
    1006: About;
  end;
  ClearEvent(Event);
end;

procedure TMainApp.Help;
  var R: TRect;
begin
  if Message(Desktop, evBroadcast, 1007, nil)=nil then begin
    R.Assign(0,0,72,15);
    T := New(PHelpWindow, Init(R, 'readme.doc', ''));
    T^.Options := T^.Options or ofCentered;
    Desktop^.Insert(T);
  end
  else begin
    T^.Select;
  end;
end;

constructor TMainApp.Init;
  var R: TRect;
begin
  inherited Init;
  GetExtent(R);
  Dec(R.B.X);
  R.A.X := R.B.X - 8;
  R.A.Y := R.B.Y - 1;
  Clock := New(PClockView, Init(R));
  Insert(Clock);

  with Pntr do begin
    Size:=0;
    Mode:=1;
    OutMode:=0;
    Name:='input';
    OutName:='output';
    F:=0;
  end;

  About;
  ReadData;
  SetSizes(Sizes[Pntr.Size+1]);
end;

procedure TMainApp.ConvertData;
  var i, j, Code: Integer;
begin
  for i:=1 to X do
    for j:=1 to Y do
      Val(InputData.S[(j-1)*X+i], Data[i][j], Code);

  for j:=1 to X do
    Val(InputData.S[(Y-1)*X+j], Fm[j], Code);
end;

procedure TMainApp.ReadData;
  var f: Text;
      i: Integer;
begin
  if Pntr.Mode=1 then begin
    Assign(f, Pntr.Name+'.txt');
    Reset(f);
    {$I-}
    for i:=1 to 20 do
      Readln(f, InputData.S[i]);
    Close(f);
    {$I+}
  end;
end;

procedure TMainApp.SetSizes(Siz: Byte); assembler;
asm
  mov al, Siz
  shr al, 4
  mov X, al
  mov al, Siz
  and al, 15
  mov Y, al
end;

var Simplex: TMainApp;

begin
  Simplex.Init;
  Simplex.Run;
  Simplex.Done;
end.
